## 介绍
　　基础库提供了丰富的Java工具包，它能够帮助我们简化每一行代码（集成[Hutool](https://hutool.cn)工具包）。<br>
　　同时也自动装配了一系列的基础Bean，可在<b>application.yml</b>文件中配置关闭，所有配置项皆是以`yue.*`开头，如：`yue.cors.allow=false`代表不允许跨域。
- 丰富的Java基础工具类，对文件、流、加密解密、转码、正则、线程、XML等JDK方法进行封装
- 默认开启热加载、热部署与跨域支持，并提供增强的Controller方法参数解析器，让前后端联调和平相处
- 全局统一异常处理基类，结合`Result`对象，定位异常更轻松，前端显示更贴切
- 异步线程池：共用父线程上下文环境，异步执行任务时不丢失token
- `Result`Http最外层响应对象，更适应Restful风格API
- `validator`参数校验器，支持单参数连写与POJO对象（注解）校验等，更多的校验规则，更贴切的国内校验场景。（如：手机号、身份证号码）
- `Convert`类型转换器，内置hutool、fastjson、yue三种类型转换规则，判断精确性能强大，未知类型兼容性更强

## 配置项
　　`yue-library-base`自动装配了一系列的基础Bean与环境配置项，可在<b>application.yml</b>文件中配置关闭，所有配置项皆是以`yue.*`开头，如：`yue.cors.allow=false`代表不允许跨域。
- Validator-校验器 （唯一Bean，可直接覆盖）
- RestTemplate-HTTPS客户端 （唯一Bean，可直接覆盖）
- ExceptionHandlerConfig-全局统一异常处理 `yue.exception-handler.*`
- AsyncConfig-异步线程池（共用父线程上下文环境，异步执行任务时不丢失token） `yue.thread-pool.async.*`
- CorsConfig-跨域配置 `yue.cors.allow`

　　在 `application.yml` 或 `bootstrap.yml` 文件中添加如下示例配置，导入 **yue-library** 提供的默认配置支持。如：时间格式化、可执行SQL打印、接口定义打印等
```yml
spring:
  profiles:
    include:
	- yue-library-base
    - yue-library-web
	- yue-library-webflux
    - yue-library-data-jdbc
	- yue-library-data-redis
	...
```

## 特性介绍
### Restful
　　`ai.yue.library.base.view.Result<T>` 定义为最外层响应对象，`ai.yue.library.base.view.ResultInfo` 定义为工具类可便捷返回 Restful 风格API结果。<br>
　　`@ApiVersion` 注解可优雅的实现API接口版本控制，如：https://ylyue.cn/v1.2/user

### 统一异常处理
　　`ai.yue.library.base.config.handler.ExceptionHandlerConfig` 全局统一异常处理配置，将拦截 Controller 层抛出的所有异常进行统一处理并响应相应的提示（包括 Restful 风格的异常提示）

### 工具类
　　依托于 [Hutool](https://hutool.cn) 工具包等，进行校正与增强，简单强大只有你想不到。

### 参数解析器
　　Controller方法参数解析器，解决参数获取困扰，不再区分Query传参与Body传参，Request请求参数智能解析。只要前端同学传了你想要的参数，就能获取得到，让前后端联调和平相处。

### 类型转换器
　　提供简单全面的类型转换，适合更多的业务场景，内置hutool、fastjson、yue三种类型转换器，判断精确性能强大，未知类型兼容性更强<br>
　　[类型转换工具类-Convert参考](https://hutool.cn/docs/#/core/类型转换/类型转换工具类-Convert)

### 校验
　　全面简单的校验框架，更适合国内校验场景，支持多种校验方式，配合全局异常处理，只为更好的`Restful`<br>
　　[validator使用文档](base/校验.md)

## 文档说明
　　基础工具包文档见：[`hutool-core`](https://hutool.cn/docs)<br>
　　更多工具包文档见本示例说明：[工具类](base/工具类.md)，[校验](base/校验.md) ...