# 参数解析器
Controller方法参数解析器，解决参数获取困扰，不再区分Query传参与Body传参，Request请求参数智能解析。只要前端同学传了你想要的参数，就能获取得到，让前后端联调和平相处。

如：后端同学写API文档使用的是类似于Postman的工具，为快速描述接口需要哪些参数，基本选择将参数填写在`form-data`框中，而实际调用接口的前端同学却采用的是另外一种方式对参数进行提交，如：`Content-Type=application/json`，此时便需要对SpringBoot的参数解析器进行增强处理，才可兼容不同的传参方式。

> 基于yue-library的项目，依托于此类强大的参数解析器，联调苦难不再，从此头发茂密如林，乌黑发亮

## 参数解析工具类
```java
ParamUtils.getParam()
ParamUtils.getParam(Class<T>)
```

## 基本参数解析器
- 不再区分Query传参与Body传参
- 时间类型反序列化：进行增强解析以处理更多时间格式
- 时间类型序列化：进行统一格式化（yyyy-MM-dd HH:mm:ss）

目前已测试通过如下几种时间格式：
```java
java.util.Date
java.time.LocalDate
java.time.LocalTime
java.time.LocalDateTime
cn.hutool.core.date.DateTime
```

基本参数解析：
```java
@PostMapping
public Result<?> post(String v1, int v2, boolean v3, v4 v5 ...) {
```

## JSONObject参数解析器
优先级最高的参数解析器，常用于 `Controller JSON传参 → Service校验 → DAO智能解析` 的快速开发模式

```java
@PostMapping
public Result<?> post(JSONObject paramJson) {
```

## JavaBean参数解析器
此参数解析器优先级高于Spring提供的ServletModelAttributeMethodProcessor参数解析器，专门作用于POJO类处理

```java
@PostMapping
public Result<?> post(ValidationIPO validationIPO) {
```